/* The 15-410 kernel project
 *
 *     loader.h
 *
 * Structure definitions, #defines, and function prototypes
 * for the user process loader.
 */

#ifndef _LOADER_H
#define _LOADER_H

/* --- The a.out header struct --- */
typedef struct exec {
  unsigned long   a_midmag; /* The magic number */
  unsigned long   a_text;   /* length of text section */
  unsigned long   a_data;   /* length of data section */
  unsigned long   a_bss;    /* length of bss  section */
  unsigned long   a_syms;   /* length of symbol table in bytes */
  unsigned long   a_entry;  /* entry point */
  unsigned long   a_trsize; /* length of relocation info for text */
  unsigned long   a_drsize; /* length of relocation info for data */
} exec_t;
 
/* --- Simplified ELF header --- */
typedef struct simple_elf {
  const char *  e_fname;       /* filename of binary */
  unsigned long e_entry;       /* entry point virtual address */
  unsigned long e_txtoff;      /* offset of text segment in file */
  unsigned long e_txtlen;      /* length of text segment in bytes */
  unsigned long e_txtstart;    /* start of text segment virtual address */
  unsigned long e_datoff;      /* offset of data segment in file */
  unsigned long e_datlen;      /* length of data segment in bytes */
  unsigned long e_datstart;    /* start of data segment in virtual memory */
  unsigned long e_rodatoff;    /* offset of rodata segment in file */
  unsigned long e_rodatlen;    /* length of rodata segment in bytes */
  unsigned long e_rodatstart;  /* start of rodata segment in virtual memory*/
  unsigned long e_bsslen;      /* length of bss  segment in bytes */
} simple_elf_t;
     
/* --- Defines --- */
#define NMAGIC 0410 /* 0x108 */
#define N_MAGIC(exec) ((exec).a_midmag & 0xffff)
#define N_BADMAG(x) (N_MAGIC(x) != NMAGIC)
  
#define ELF_NOTELF  -1
#define ELF_SUCCESS  0
#define NOT_PRESENT  0


/* --- Prototypes --- */

/*
 * Declare your loader prototypes here.
 */

#endif /* _LOADER_H */
